@@ -4,7 +4,6 @@ from joule import utilities
 import asyncio
 import numpy as np
 import textwrap
-import argparse
 
 ARGS_DESC = """
 ---
@@ -57,8 +56,10 @@ ARGS_DESC = """
     exec_cmd = joule modules random-reader
 
     [Arguments]
+    # number of elements
     width = 4
-    rate = 10 #Hz
+    # data rate in Hz
+    rate = 10 
 
     [Outputs]
     output = /path/to/output
@@ -70,14 +71,16 @@ OUTPUT_RATE = 1  # run in 1 second blocks
 
 class RandomReader(ReaderModule):
     """Generate a random stream of data"""
-    
-    def custom_args(self, parser: argparse.ArgumentParser):  # pragma: no cover
-        parser.add_argument("--width", type=int,
-                            required=True,
-                            help="number of elements in output")
-        parser.add_argument("--rate", type=float,
-                            required=True,
-                            help="rate in Hz")
+
+    def custom_args(self, parser):  # pragma: no cover
+        grp = parser.add_argument_group("module",
+                                        "module specific arguments")
+        grp.add_argument("--width", type=int,
+                         required=True,
+                         help="number of elements in output")
+        grp.add_argument("--rate", type=float,
+                         required=True,
+                         help="rate in Hz")
         parser.description = textwrap.dedent(ARGS_DESC)
 
     async def run(self, parsed_args, output):
@@ -91,6 +94,7 @@ class RandomReader(ReaderModule):
         BLOCK_SIZE = rate/OUTPUT_RATE
         fraction_remaining = 0
         i = 0
+
         # print("Starting random stream: %d elements @ %0.1fHz" % (width, rate))
         while not self.stop_requested:
             float_block_size = BLOCK_SIZE+fraction_remaining
