@@ -11,7 +11,7 @@ from . import base_module
 class CompositeModule(base_module.BaseModule):
 
     async def setup(self, parsed_args: argparse.Namespace, inputs: Dict[str, Pipe],
-                    outputs: Dict[str, Pipe], loop: asyncio.AbstractEventLoop):
+                    outputs: Dict[str, Pipe]):
         """
         This method must be implemented
 
@@ -28,14 +28,13 @@ class CompositeModule(base_module.BaseModule):
 
         assert False, "implement in child class"  # pragma: no cover
 
-    async def run_as_task(self, parsed_args, app, loop) -> asyncio.Task:
+    async def run_as_task(self, parsed_args, app) -> asyncio.Task:
         try:
             (pipes_in, pipes_out) = await self._build_pipes(parsed_args)
         except errors.ApiError as e:
             logging.error(str(e))
-            return loop.create_task(asyncio.sleep(0))
+            return asyncio.create_task(asyncio.sleep(0))
         tasks = await self.setup(parsed_args,
                                  pipes_in,
-                                 pipes_out,
-                                 loop)
+                                 pipes_out)
         return asyncio.gather(*tasks)
