@@ -5,7 +5,8 @@ import asyncio
 import re
 import numpy as np
 from typing import List, Dict, Optional, Callable, Coroutine
-from joule.models import Stream, pipes
+from joule.utilities import interval_tools, timestamp_to_datetime
+from joule.models import DataStream, pipes
 from joule.models.data_store.data_store import DataStore, StreamInfo, DbInfo, Interval
 from joule.models.data_store import errors
 from joule.models.data_store.nilmdb_inserter import Inserter
@@ -22,15 +23,15 @@ class PathNotFound(Exception):
 
 class NilmdbStore(DataStore):
 
-    def __init__(self, server: str, insert_period: float, cleanup_period: float, loop: Loop):
+    def __init__(self, server: str, insert_period: float, cleanup_period: float):
         self.server = server
         self.decimation_factor = 4
         self.insert_period = insert_period
         self.cleanup_period = cleanup_period
-        self.connector = aiohttp.TCPConnector(loop=loop)
-        self.loop = loop
+        self.connector = None
 
-    async def initialize(self, streams: List[Stream]) -> None:
+    async def initialize(self, streams: List[DataStream]) -> None:
+        self.connector = aiohttp.TCPConnector()
         url = "{server}/stream/create".format(server=self.server)
         try:
             async with self._get_client() as session:
@@ -42,7 +43,7 @@ class NilmdbStore(DataStore):
         except aiohttp.ClientError:
             raise errors.DataError("cannot contact NilmDB at [%s]" % self.server)
 
-    async def insert(self, stream: Stream, start: int, end: int, data: np.array) -> None:
+    async def insert(self, stream: DataStream, start: int, end: int, data: np.array) -> None:
         """insert stream data"""
         url = "{server}/stream/insert".format(server=self.server)
         params = {"start": "%d" % start,
@@ -51,7 +52,7 @@ class NilmdbStore(DataStore):
                   "binary": '1'}
         async with self._get_client() as session:
             async with session.put(url, params=params,
-                                   data=data.tostring()) as resp:
+                                   data=data.tobytes()) as resp:
                 if resp.status != 200:
                     if resp.status == 400:
                         error = await resp.json()
@@ -59,16 +60,18 @@ class NilmdbStore(DataStore):
                         raise errors.DataError(error["message"])
                     raise errors.DataError(await resp.text())  # pragma: no cover
 
-    async def spawn_inserter(self, stream: Stream,
-                             pipe: pipes.Pipe, loop: Loop, insert_period=None, retry_interval=0.5) -> asyncio.Task:
+    async def spawn_inserter(self, stream: DataStream,
+                             pipe: pipes.Pipe, insert_period=None, retry_interval=0.5) -> asyncio.Task:
         if insert_period is None:
             insert_period = self.insert_period
         inserter = Inserter(self.server, stream,
                             insert_period, self.cleanup_period, self._get_client,
                             retry_interval=retry_interval)
-        return loop.create_task(inserter.run(pipe, loop))
+        t = asyncio.create_task(inserter.run(pipe))
+        t.set_name("NilmDB Inserter for [%s]" % stream.name)
+        return t
 
-    async def extract(self, stream: Stream, start: Optional[int], end: Optional[int],
+    async def extract(self, stream: DataStream, start: Optional[int], end: Optional[int],
                       callback: Callable[[np.ndarray, str, int], Coroutine], max_rows: int = None,
                       decimation_level=None):
         # figure out appropriate decimation level
@@ -118,7 +121,7 @@ class NilmdbStore(DataStore):
         except aiohttp.ClientError as e:
             raise errors.DataError(str(e))
 
-    async def intervals(self, stream: Stream, start: Optional[int], end: Optional[int]):
+    async def intervals(self, stream: DataStream, start: Optional[int], end: Optional[int]):
         try:
             return await self._intervals_by_path(compute_path(stream),
                                                  start, end)
@@ -129,11 +132,42 @@ class NilmdbStore(DataStore):
             else:
                 raise e
 
+    async def consolidate(self, stream: 'DataStream', start: int, end: int, max_gap: int) -> int:
+        # remove interval gaps less than or equal to max_gap duration (in us)
+        intervals = await self.intervals(stream, start, end)
+        if len(intervals) == 0:
+            return  # no data, nothing to do
+        duration = [intervals[0][0], intervals[-1][1]]
+
+        gaps = interval_tools.interval_difference([duration], intervals)
+
+        if len(gaps) == 0:
+            return  # no interval breaks, nothing to do
+        small_gaps = [gap for gap in gaps if (gap[1] - gap[0]) <= max_gap]
+        # spawn an inserter to close each gap
+
+        info = await self._path_info([stream])
+        all_paths = info.keys()
+        path = compute_path(stream)
+        insert_url = "{server}/stream/insert".format(server=self.server)
+        for gap in small_gaps:
+            async with self._get_client() as session:
+                params = {"start": "%d" % gap[0],
+                          "end": "%d" % gap[1],
+                          "path": path,
+                          "binary": '1'}
+                async with session.put(insert_url, params=params,
+                                       data=None) as resp:
+                    if resp.status != 200:  # pragma: no cover
+                        error = await resp.text()
+                        raise errors.DataError("NilmDB(d) error: %s" % error)
+        return len(small_gaps)
+
     # TODO: remove path lookup, iterate until the stream decimation isn't found
     async def remove(self, stream, start: Optional[int] = None,
                      end: Optional[int] = None):
         """ remove [start,end] in path and all decimations """
-        info = await self._path_info()
+        info = await self._path_info([stream])
         all_paths = info.keys()
         base_path = compute_path(stream)
         regex = re.compile(r"%s~decim-(\d)+$" % base_path)
@@ -141,14 +175,16 @@ class NilmdbStore(DataStore):
         for path in [base_path, *decim_paths]:
             await self._remove_by_path(path, start, end)
 
-    async def info(self, streams: List[Stream]) -> Dict[int, StreamInfo]:
-        info_dict = await self._path_info()
-        # go through each stream and compute the effective StreamInfo
+    async def info(self, streams: List[DataStream]) -> Dict[int, StreamInfo]:
+        info_dict = await self._path_info(streams)
+        # go through each stream and compute the effective DataStreamInfo
         # object by looking at the raw and decimated paths
         streams_info = {}
         for s in streams:
             base_info_list = [info for (path, info) in info_dict.items() if path == compute_path(s)]
             if len(base_info_list) == 0:
+                # this stream has no data records, just make up an empty result
+                streams_info[s.id] = StreamInfo(None, None, 0, 0, 0)
                 continue
             stream_info: StreamInfo = base_info_list[0]
             if stream_info.rows != 0:
@@ -172,10 +208,10 @@ class NilmdbStore(DataStore):
                 data = await resp.json()
                 return DbInfo(**data)
 
-    async def destroy(self, stream: Stream):
+    async def destroy(self, stream: DataStream):
         await self.remove(stream)
         url = "{server}/stream/destroy".format(server=self.server)
-        info = await self._path_info()
+        info = await self._path_info([stream])
         all_paths = info.keys()
         base_path = compute_path(stream)
         regex = re.compile(r'%s~decim-(\d)+$' % base_path)
@@ -186,6 +222,9 @@ class NilmdbStore(DataStore):
                     await check_for_error(resp, ignore=[ERRORS.NO_STREAM_AT_PATH])
         return web.Response(text="ok")
 
+    async def destroy_all(self):
+        raise Exception("NilmDB backend does not implement erase_all")
+
     async def _extract_by_path(self, path: str, start: Optional[int], end: Optional[int],
                                layout: str, callback):
         url = "{server}/stream/extract".format(server=self.server)
@@ -196,8 +235,14 @@ class NilmdbStore(DataStore):
         if r is not None:
             decimation_factor = int(r[1])
         async with self._get_client() as session:
-            # first determine the intervals
-            intervals = await self._intervals_by_path(path, start, end)
+            # first determine the intervals, use the base path for this
+            if path.find("~decim") == -1:
+                base_path = path
+            else:
+                base_path = path[:path.find("~decim")]
+            intervals = await self._intervals_by_path(base_path, start, end)
+            i = 0
+            num_intervals = len(intervals)
             # now extract each interval
             for interval in intervals:
                 params["start"] = interval[0]
@@ -214,11 +259,14 @@ class NilmdbStore(DataStore):
                         except pipes.EmptyPipe:
                             break
                 # insert the interval token to indicate a break
-                await callback(pipes.interval_token(layout), layout, decimation_factor)
+                i += 1
+                if i < num_intervals:
+                    await callback(pipes.interval_token(layout), layout, decimation_factor)
 
     async def _intervals_by_path(self, path: str, start: Optional[int],
                                  end: Optional[int]) -> List[Interval]:
         url = "{server}/stream/intervals".format(server=self.server)
+        base_path = path
         params = {"path": path}
         if start is not None:
             params["start"] = start
@@ -233,7 +281,8 @@ class NilmdbStore(DataStore):
                     return []
                 for line in text.strip().split("\n"):
                     intervals.append(json.loads(line))
-        return intervals
+
+        return consolidate_intervals(intervals)
 
     async def _remove_by_path(self, path: str, start: Optional[int], end: Optional[int]):
         """remove data from streams"""
@@ -261,10 +310,17 @@ class NilmdbStore(DataStore):
                 await check_for_error(resp)
                 return int(await resp.text())
 
-    async def _path_info(self) -> Dict[str, StreamInfo]:
+    async def _path_info(self, streams: List[DataStream]) -> Dict[str, StreamInfo]:
         """ set path to None to list all streams """
         url = "{server}/stream/list".format(server=self.server)
-        params = {"extended": 1}
+        paths = []
+        if len(streams) > 6:
+            paths = "/joule/%"
+        else:
+            for stream in streams:
+                paths += [f'/joule/{stream.id}', f'/joule/{stream.id}~decim%']
+        params = {"extended": 1,
+                  "path": paths}
         async with self._get_client() as session:
             async with session.get(url, params=params) as resp:
                 body = await resp.text()
@@ -281,11 +337,11 @@ class NilmdbStore(DataStore):
                 return info
 
     def _get_client(self):
-        return aiohttp.ClientSession(connector=self.connector,
-                                     loop=self.loop, connector_owner=False)
+        return aiohttp.ClientSession(connector=self.connector, connector_owner=False)
 
     async def close(self):
-        self.connector.close()
+        if self.connector is not None:
+            await self.connector.close()
 
 
 def bytes_per_row(layout: str) -> int:
@@ -302,3 +358,20 @@ def bytes_per_row(layout: str) -> int:
             raise ValueError("bad layout %s" % layout)
     except (ValueError, IndexError):
         raise ValueError("bad layout: %s" % layout)
+
+
+def consolidate_intervals(intervals):
+    # consolidate intervals that touch
+    consolidated_intervals = []
+    if len(intervals) <= 1:
+        return intervals
+    prev_interval = intervals[0]
+    for i in range(1, len(intervals)):
+        if intervals[i][0] == prev_interval[1]:
+            # extend prev_interval
+            prev_interval[1] = intervals[i][1]
+        else:
+            consolidated_intervals.append(prev_interval)
+            prev_interval = intervals[i]
+    consolidated_intervals.append(prev_interval)
+    return consolidated_intervals
