@@ -3,10 +3,11 @@ import aiohttp
 import numpy as np
 import random
 import time
-from typing import List, Callable
+from typing import List, Callable, Optional
 import logging
+from icecream import ic
 
-from joule.models import Stream, pipes
+from joule.models import DataStream, pipes
 from joule.models.data_store import errors
 from joule.models.data_store.nilmdb_helpers import compute_path, ERRORS, check_for_error
 
@@ -16,7 +17,7 @@ log = logging.getLogger('joule')
 
 class Inserter:
 
-    def __init__(self, server: str, stream: Stream, insert_period: float, cleanup_period: float,
+    def __init__(self, server: str, stream: DataStream, insert_period: float, cleanup_period: float,
                  session_factory: Callable[[], aiohttp.ClientSession], retry_interval=0.5):
         self.insert_url = "{server}/stream/insert".format(server=server)
         self.remove_url = "{server}/stream/remove".format(server=server)
@@ -32,7 +33,7 @@ class Inserter:
         self.decimator = None
         self.retry_interval = retry_interval
 
-    async def run(self, pipe: pipes.Pipe, loop: Loop) -> None:
+    async def run(self, pipe: pipes.Pipe) -> None:
         """insert stream data from the queue until the queue is empty"""
         # create the database path
         # lazy stream creation,
@@ -41,10 +42,10 @@ class Inserter:
         except asyncio.CancelledError:
             return
 
-        cleaner_task: asyncio.Task = None
-        if self.stream.keep_us != Stream.KEEP_ALL:
-            cleaner_task = loop.create_task(self._clean())
-
+        cleaner_task: Optional[asyncio.Task] = None
+        if self.stream.keep_us != DataStream.KEEP_ALL:
+            cleaner_task = asyncio.create_task(self._clean())
+            cleaner_task.set_name("NilmDB Clean Task for [%s]" % self.path)
         while True:
             try:
                 async with self._get_client() as session:
@@ -70,7 +71,7 @@ class Inserter:
                                       "path": self.path,
                                       "binary": '1'}
                             async with session.put(self.insert_url, params=params,
-                                                   data=data.tostring()) as resp:
+                                                   data=data.tobytes()) as resp:
                                 if resp.status != 200:
                                     error = await resp.text()
                                     if cleaner_task is not None:
@@ -90,7 +91,7 @@ class Inserter:
             except aiohttp.ClientError as e:  # pragma: no cover
                 log.warning("NilmDB raw inserter error: %r, retrying request" % e)
                 await asyncio.sleep(self.retry_interval)  # retry the request
-            except (pipes.EmptyPipe, asyncio.CancelledError):
+            except (pipes.EmptyPipe, asyncio.CancelledError) as e:
                 break  # terminate the inserter
         if cleaner_task is not None:
             cleaner_task.cancel()
@@ -140,7 +141,7 @@ class Inserter:
 
 class NilmdbDecimator:
 
-    def __init__(self, server: str, stream: Stream, from_level: int, factor: int,
+    def __init__(self, server: str, stream: DataStream, from_level: int, factor: int,
                  session_factory: Callable[[], aiohttp.ClientSession], retry_interval=0.5):
         self.stream = stream
         self.level = from_level * factor
@@ -190,7 +191,7 @@ class NilmdbDecimator:
                               "path": self.path,
                               "binary": '1'}
                     async with session.put(self.insert_url, params=params,
-                                           data=decim_data.tostring()) as resp:
+                                           data=decim_data.tobytes()) as resp:
                         if resp.status != 200:  # pragma: no cover
                             error = await resp.text()
                             raise errors.DataError("NilmDB(d) error: %s" % error)
