@@ -6,7 +6,7 @@ import logging
 from joule.models.pipes.errors import PipeError
 
 if TYPE_CHECKING:  # pragma: no cover
-    from joule.models import (Module, Stream)
+    from joule.models import (Module, DataStream)
 
 log = logging.getLogger('joule')
 
@@ -40,7 +40,7 @@ class Pipe:
         self.name: str = name
         self.direction: Pipe.DIRECTION = direction
         self.module: 'Module' = module
-        self.stream: 'Stream' = stream
+        self.stream: 'DataStream' = stream
         self._layout = layout
         self.closed = False
         self.decimation_level = 1
@@ -177,6 +177,18 @@ class Pipe:
             raise PipeError("cannot consume from an output pipe")
         raise PipeError("abstract method must be implemented by child")
 
+    def consume_all(self):
+        """
+        Flush all data from the read buffer. The next call to :meth:`read` will
+        only return new incoming data.
+        """
+        if self.direction == Pipe.DIRECTION.OUTPUT:
+            raise PipeError("cannot consume from an output pipe")
+        raise PipeError("abstract method must be implemented by child")
+
+    def is_empty(self):
+        raise PipeError("not implemented")
+
     def fail(self):
         self._failed = True
 
@@ -285,7 +297,7 @@ class Pipe:
             try:
                 sarray['timestamp'] = data[:, 0]
                 # Need the squeeze in case sarray['data'] is 1 dimensional
-                sarray['data'] = np.squeeze(data[:, 1:])
+                sarray['data'] = data[:, 1:]
                 return sarray
             except (IndexError, ValueError):
                 raise PipeError("wrong number of fields for this data type")
@@ -352,6 +364,6 @@ def compute_dtype(layout: str) -> np.dtype:
             atype = '<f' + str(int(ltype[5:]) // 8)
         else:
             raise ValueError()
-        return np.dtype([('timestamp', '<i8'), ('data', atype, lcount)])
+        return np.dtype([('timestamp', '<i8'), ('data', atype, (lcount,))])
     except (IndexError, ValueError):
         raise ValueError("bad layout: [%s]" % layout)
